@@ -61,9 +61,11 @@ class IndexController extends \OPNsense\Base\IndexController
         $this->view->sni_hostname_map = $this->getForm("sni_hostname_map");
         $this->view->ipacl = $this->getForm("ipacl");
         $this->view->tls_fingerprint = $this->getForm("tls_fingerprint");
+        $this->view->syslog_target = $this->getForm("syslog_target");
         $nginx = new Nginx();
         $this->view->show_naxsi_download_button =
-            count($nginx->custom_policy->iterateItems()) == 0 && count($nginx->naxsi_rule->iterateItems()) == 0;
+            count(iterator_to_array($nginx->custom_policy->iterateItems())) == 0 &&
+            count(iterator_to_array($nginx->naxsi_rule->iterateItems())) == 0;
         $this->view->pick('OPNsense/Nginx/index');
     }
 
@@ -90,4 +92,12 @@ class IndexController extends \OPNsense\Base\IndexController
     {
         $this->view->pick('OPNsense/Nginx/ban');
     }
+
+    /**
+     * display a viewer for traffic statistics.
+     */
+    public function vtsAction()
+    {
+        $this->view->pick('OPNsense/Nginx/vts');
+    }
 }
