@@ -2,7 +2,7 @@
 <?php
 
 /*
- * Copyright (C) 2017-2018 Frank Wall
+ * Copyright (C) 2017-2019 Frank Wall
  * Copyright (C) 2015 Deciso B.V.
  * Copyright (C) 2010 Jim Pingle <jimp@pfsense.org>
  * Copyright (C) 2008 Shrew Soft Inc. <mgrooms@shrew.net>
@@ -77,20 +77,25 @@ if (isset($options["C"])) {
 // Run the specified action
 switch ($options["a"]) {
     case 'sign':
-        //$result = sign_or_renew_cert($options["c"]);
         $result = cert_action_validator($options["c"]);
         echo json_encode(array('status'=>$result));
         break;
     case 'renew':
-        //$result = sign_or_renew_cert($options["c"]);
+        $result = cert_action_validator($options["c"]);
+        echo json_encode(array('status'=>$result));
+        break;
+    case 'remove':
+        $result = cert_action_validator($options["c"]);
+        echo json_encode(array('status'=>$result));
+        break;
+    case 'removekey':
         $result = cert_action_validator($options["c"]);
         echo json_encode(array('status'=>$result));
         break;
     case 'revoke':
-        //$result = revoke_cert($options["c"]);
         $result = cert_action_validator($options["c"]);
         echo json_encode(array('status'=>$result));
-        exit(1);
+        break;
     default:
         echo "ERROR: invalid argument specified\n";
         log_error("invalid argument specified");
@@ -126,10 +131,14 @@ function cert_action_validator($opt_cert_id)
             if (isset($options["A"]) or ((string)$cert_id == (string)$opt_cert_id)) {
                 // Ignore disabled certificates
                 if ($certObj->enabled == 0) {
+                    // Always ignore disabled certs when working on ALL certs.
                     if (isset($options["A"])) {
                         continue; // skip to next item
                     }
-                    return(1); // Cert is disabled, skip it.
+                    // Allow only "revoke", "remove" and "removekey" for disabled certs.
+                    if (!in_array($options["a"], ['remove','removekey','revoke'])) {
+                        return(1); // Cert is disabled, skip it.
+                    }
                 }
 
                 // Extract Account from referenced obj
@@ -179,6 +188,37 @@ function cert_action_validator($opt_cert_id)
                     }
                 }
 
+                // Cert is being removed from the GUI, delete all traces.
+                if ($options["a"] == "remove") {
+                    // Start acme client to remove the certificate
+                    $rev_result = remove_cert($certObj);
+                    if (!$rev_result) {
+                        log_error("AcmeClient: successfully removed acme.sh certificate configuration for " . (string)$certObj->name);
+                        return(0); // Success!
+                    } else {
+                        log_error("AcmeClient: failed to remove acme.sh certificate configuration for " . (string)$certObj->name);
+                        return(1);
+                    }
+                }
+
+                // Remove private key
+                // NOTE: Although the user requested to remove the private key,
+                // we simply perform a full cert removal because without the
+                // matching private key the cert is useless.
+                if ($options["a"] == "removekey") {
+                    // Start acme client to remove the certificate
+                    $rev_result = remove_cert($certObj);
+                    if (!$rev_result) {
+                        log_error("AcmeClient: successfully removed the private key and reset certificate " . (string)$certObj->name);
+                        // Reset certificate state, treat it like a new certificate.
+                        log_cert_acme_status($certObj, $modelObj, '100');
+                        return(0); // Success!
+                    } else {
+                        log_error("AcmeClient: failed to remove the private key and reset certificate " . (string)$certObj->name);
+                        return(1);
+                    }
+                }
+
                 // Make sure we found the configured validation method
                 if ($ref_found == true) {
                     // Was a revocation requested?
@@ -206,7 +246,7 @@ function cert_action_validator($opt_cert_id)
                         // Start acme client to issue or renew certificate
                         $val_result = run_acme_validation($certObj, $valObj, $acctObj);
                         if (!$val_result) {
-                            log_error("AcmeClient: issued/renewed certificate: " . (string)$certObj->name);
+                            log_error("AcmeClient: successfully issued/renewed certificate: " . (string)$certObj->name);
                             // Import certificate to Cert Manager
                             if (!import_certificate($certObj, $modelObj)) {
                                 // Prepare certificate for automation
@@ -371,6 +411,7 @@ function run_acme_account_registration($acctObj, $certObj, $modelObj)
             // serialize to config and save
             $modelObj->serializeToConfig();
             Config::getInstance()->save();
+            Config::getInstance()->forceReload();
         }
     }
 
@@ -405,6 +446,7 @@ function run_acme_account_registration($acctObj, $certObj, $modelObj)
         // serialize to config and save
         $modelObj->serializeToConfig();
         Config::getInstance()->save();
+        Config::getInstance()->forceReload();
     }
 
     return;
@@ -417,6 +459,7 @@ function run_acme_validation($certObj, $valObj, $acctObj)
 
     // Required to run pre-defined commands.
     $backend = new Backend();
+    $modelObj = new OPNsense\AcmeClient\AcmeClient;
 
     // Collect account information
     $account_conf_dir = "/var/etc/acme-client/accounts/" . $acctObj->id;
@@ -567,6 +610,7 @@ function run_acme_validation($certObj, $valObj, $acctObj)
     // Prepare DNS-01 hooks
     if ($val_method == 'dns01') {
         // Some common stuff
+        $val_id = preg_replace("/[^a-zA-Z0-9]/", "", (string)$valObj->id);
         $secret_key_filename = "${configdir}/secret.key";
         $acme_args[] = '--dnssleep ' . $valObj->dns_sleep;
 
@@ -626,6 +670,9 @@ function run_acme_validation($certObj, $valObj, $acctObj)
                 $proc_env['DO_PID'] = (string)$valObj->dns_do_pid;
                 $proc_env['DO_PW'] = (string)$valObj->dns_do_password;
                 break;
+            case 'dns_doapi':
+                $proc_env['DO_LETOKEN'] = (string)$valObj->dns_doapi_token;
+                break;
             case 'dns_dp':
                 $proc_env['DP_Id'] = (string)$valObj->dns_dp_id;
                 $proc_env['DP_Key'] = (string)$valObj->dns_dp_key;
@@ -652,10 +699,50 @@ function run_acme_validation($certObj, $valObj, $acctObj)
             case 'dns_gandi_livedns':
                 $proc_env['GANDI_LIVEDNS_KEY'] = (string)$valObj->dns_gandi_livedns_key;
                 break;
+            case 'dns_gcloud':
+                # Google Cloud SDK must be installed.
+                if ((string)$modelObj->isPluginInstalled('google-cloud-sdk') != "1") {
+                    log_error("AcmeClient: Google Cloud SDK plugin is NOT installed. Please install os-google-cloud-sdk.");
+                    return(1);
+                }
+                # We need a valid Google Cloud JSON key.
+                if (!empty((string)$valObj->dns_gcloud_key)) {
+                    # Extract the gcloud project from the key data.
+                    $_gcloud_data = json_decode((string)$valObj->dns_gcloud_key);
+                    $gcloud_project = $_gcloud_data->project_id;
+                    $gcloud_account = $_gcloud_data->client_email;
+                    if (empty($gcloud_project)) {
+                        log_error("AcmeClient: unable to extract project name from Google Cloud DNS JSON key");
+                        return(1);
+                    } else {
+                        log_error("AcmeClient: Google Cloud DNS project name: ${gcloud_project}");
+                    }
+                } else {
+                    log_error("AcmeClient: no key for Google Cloud DNS was specified");
+                    return(1);
+                }
+                # Preparations for gcloud CLI.
+                $gcloud_config = "acme-${val_id}";
+                $gcloud_key_file = "/tmp/acme_" . (string)$valObj->dns_service . "_${val_id}.json";
+                file_put_contents($gcloud_key_file, (string)$valObj->dns_gcloud_key);
+                chmod($gcloud_key_file, 0600);
+                $proc_env['CLOUDSDK_ACTIVE_CONFIG_NAME'] = $gcloud_config;
+                $proc_env['CLOUDSDK_CORE_PROJECT'] = $gcloud_project;
+                # Ensure that a working gcloud config exists.
+                run_shell_command("/usr/local/bin/gcloud config configurations create ${gcloud_config}", $proc_env);
+                run_shell_command("/usr/local/bin/gcloud config configurations activate ${gcloud_config}", $proc_env);
+                run_shell_command("/usr/local/bin/gcloud auth activate-service-account --key-file=${gcloud_key_file}", $proc_env);
+                run_shell_command("/usr/local/bin/gcloud config set account ${gcloud_account}", $proc_env);
+                run_shell_command("/usr/local/bin/gcloud config set project ${gcloud_project}", $proc_env);
+                break;
             case 'dns_gd':
                 $proc_env['GD_Key'] = (string)$valObj->dns_gd_key;
                 $proc_env['GD_Secret'] = (string)$valObj->dns_gd_secret;
                 break;
+            case 'dns_hostingde':
+                $proc_env['HOSTINGDE_ENDPOINT'] = (string)$valObj->dns_hostingde_server;
+                $proc_env['HOSTINGDE_APIKEY'] = (string)$valObj->dns_hostingde_apiKey;
+                break;
             case 'dns_he':
                 $proc_env['HE_Username'] = (string)$valObj->dns_he_user;
                 $proc_env['HE_Password'] = (string)$valObj->dns_he_password;
@@ -705,6 +792,16 @@ function run_acme_validation($certObj, $valObj, $acctObj)
                 $proc_env['ME_Key'] = (string)$valObj->dns_me_key;
                 $proc_env['ME_Secret'] = (string)$valObj->dns_me_secret;
                 break;
+            case 'dns_namecheap':
+                $proc_env['NAMECHEAP_USERNAME'] = (string)$valObj->dns_namecheap_user;
+                $proc_env['NAMECHEAP_API_KEY'] = (string)$valObj->dns_namecheap_api;
+                if (!empty((string)$valObj->dns_namecheap_sourceip)) {
+                    $proc_env['NAMECHEAP_SOURCEIP'] = (string)$valObj->dns_namecheap_sourceip;
+                } else {
+                    // Use a public service to get our source IP for Namecheap API
+                    $proc_env['NAMECHEAP_SOURCEIP'] = 'https://ifconfig.co/ip';
+                }
+                break;
             case 'dns_namecom':
                 $proc_env['Namecom_Username'] = (string)$valObj->dns_namecom_user;
                 $proc_env['Namecom_Token'] = (string)$valObj->dns_namecom_token;
@@ -714,6 +811,13 @@ function run_acme_validation($certObj, $valObj, $acctObj)
                 // Namesilo applies changes to DNS records only every 15 minutes.
                 $acme_hook_options[] = "--dnssleep 960";
                 break;
+            case 'dns_netcup':
+                $proc_env['NC_CID'] = (string)$valObj->dns_netcup_cid;
+                $proc_env['NC_Apikey'] = (string)$valObj->dns_netcup_key;
+                $proc_env['NC_Apipw'] = (string)$valObj->dns_netcup_pw;
+                // netcup applies changes to DNS records only every 10 minutes.
+                $acme_hook_options[] = "--dnssleep 600";
+                break;
             case 'dns_nsone':
                 $proc_env['NS1_Key'] = (string)$valObj->dns_nsone_key;
                 break;
@@ -809,6 +913,7 @@ function run_acme_validation($certObj, $valObj, $acctObj)
     $acmecmd = "/usr/local/sbin/acme.sh "
       . implode(" ", $acme_args) . " "
       . "--${acme_action} "
+      . "--days " . (string)$certObj->renewInterval . " "
       . "--domain " . (string)$certObj->name . " "
       . $altnames
       . $acme_validation . " "
@@ -887,7 +992,47 @@ function revoke_cert($certObj, $valObj, $acctObj)
       . $ecc_param;
     $result = mwexec($acmecmd);
 
-    // TODO: maybe clear lastUpdate value?
+    // Simply return acme clients exit code
+    return($result);
+}
+
+// Remove a cert from list of certs known to acme.sh.
+function remove_cert($certObj)
+{
+    // Prepare optional parameters for acme-client
+    $acme_args = eval_optional_acme_args();
+
+    // Generate certificate filenames
+    $cert_id = (string)$certObj->id;
+
+    // Check if EC certificate is used, if yes add the --ecc parameter to acme client
+    $key_length = (string) $certObj->keyLength;
+    $ecc_param =  " ";
+    if ($key_length == 'key_ec256' || $key_length == 'key_ec384') {
+        $ecc_param =  "--ecc";
+    }
+
+    // Run acme client
+    $acmecmd = "/usr/local/sbin/acme.sh "
+      . implode(" ", $acme_args) . " "
+      . "--remove "
+      . "--domain " . (string)$certObj->name . " "
+      . "--home /var/etc/acme-client/home "
+      . $ecc_param;
+    $result = mwexec($acmecmd);
+
+    $cert_files = [
+        "/var/etc/acme-client/keys/${cert_id}/private.key",
+        "/var/etc/acme-client/certs/${cert_id}/cert.pem",
+        "/var/etc/acme-client/certs/${cert_id}/chain.pem",
+        "/var/etc/acme-client/certs/${cert_id}/fullchain.pem",
+    ];
+
+    foreach ($cert_files as $_file) {
+        if (file_exists($_file)) {
+            unlink($_file);
+        }
+    }
 
     // Simply return acme clients exit code
     return($result);
@@ -1064,6 +1209,7 @@ function import_certificate($certObj, $modelObj)
         // if node was found, serialize to config and save
         $modelObj->serializeToConfig();
         Config::getInstance()->save();
+        Config::getInstance()->forceReload();
     } else {
         log_error("AcmeClient: unable to update LE certificate object");
         return(1);
@@ -1198,20 +1344,62 @@ function log_cert_acme_status($certObj, $modelObj, $statusCode)
 function dump_postponed_updates()
 {
     global $postponed_updates;
+
+    $status_descr = [
+        100 => 'unknown',
+        200 => 'OK',
+        250 => 'cert revoked',
+        300 => 'configuration error',
+        400 => 'validation failed',
+        500 => 'internal error',
+    ];
+
     $modelObj = new OPNsense\AcmeClient\AcmeClient;
 
     foreach ($postponed_updates as $pupdate) {
-        $node = $modelObj->getNodeByReference('certificates.certificate.' . $pupdate['uuid']);
+        $_statusCode = $pupdate['statusCode'];
+        $_uuid = $pupdate['uuid'];
+        $node = $modelObj->getNodeByReference('certificates.certificate.'.$_uuid);
         if ($node != null) {
-            $node->statusCode = $pupdate['statusCode'];
+            log_error("AcmeClient: storing status '" . $status_descr[$_statusCode] . "' for cert " . (string)$node->name);
+            $node->statusCode = $_statusCode;
             $node->statusLastUpdate = $pupdate['statusLastUpdate'];
             // serialize to config and save
             $modelObj->serializeToConfig();
             Config::getInstance()->save();
+            Config::getInstance()->forceReload();
+        } else {
+            log_error(sprintf("AcmeClient: failed to store status '%s' for cert %s: node not found", $status_descr[$_statusCode], $_uuid));
         }
     }
 }
 
+function run_shell_command($proc_cmd, $proc_env = array())
+{
+    $proc_desc = array(  // descriptor array for proc_open()
+        0 => array("pipe", "r"), // stdin
+        1 => array("pipe", "w"), // stdout
+        2 => array("pipe", "w")  // stderr
+    );
+    $proc_pipes = array();
+    $proc = proc_open($proc_cmd, $proc_desc, $proc_pipes, null, $proc_env);
+
+    // Make sure the resource could be setup properly
+    if (is_resource($proc)) {
+        // Close all pipes
+        fclose($proc_pipes[0]);
+        fclose($proc_pipes[1]);
+        fclose($proc_pipes[2]);
+        // Get exit code
+        $result = proc_close($proc);
+        log_error(sprintf("AcmeClient: The shell command '%s' returned exit code '%d'", $proc_cmd, $result));
+        return($result);
+    } else {
+        log_error(sprintf("AcmeClient: Unable to prepare shell command '%s'", $proc_cmd));
+        return(1);
+    }
+}
+
 // taken from certs.inc
 function local_cert_get_subject_array($str_crt, $decode = true)
 {
